# This file was generated by liblab | https://liblab.com/
from __future__ import annotations
from .base import BaseModel
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .OpenAiAssistantsFunction import OpenAiAssistantsFunction


class OpenAiAssistantsToolCall(BaseModel):
    """
    Represents a tool call for OpenAI.
    """

    def __init__(self, function: OpenAiAssistantsFunction, id: str, **kwargs):
        """
        Initialize OpenAiAssistantsToolCall
        Parameters:
        ----------
            function: OpenAiFunction
            id: str
        """
        self.function = function
        self.id = id
