# This file was generated by liblab | https://liblab.com/

from .base import BaseModel


class OpenAiToolAssistantsCallResults(BaseModel):
    """
    Represents the results of a tool call for OpenAI Assistants.
    """

    def __init__(self, output: str, tool_call_id: str, **kwargs):
        """
        Initialize OpenAiToolAssistantsCallResults
        Parameters:
        ----------
            output: str
            tool_call_id: str
        """
        self.output = output
        self.tool_call_id = tool_call_id
